package cytoPrefuse;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.table.AbstractTableModel;

import prefuse.Visualization;
import prefuse.data.Tuple;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;

public class GroupsManager extends AbstractTableModel {
	private static final String ATTRIBUTE_NAME = "Cerebral groups";
	private static final String DELIMITER = "/";
	private static final long serialVersionUID = 1L;
	private ArrayList groups  = new ArrayList();
	private int currentGroup = 0;
	private int newGroupNumber = 1;
	private Visualization vis;
	private VisualItem[] selectItem = new VisualItem[1];
    private CytoPrefuse cytoPrefuse;
	
	public GroupsManager(Visualization vis) {
		this.vis =vis;
        cytoPrefuse = (CytoPrefuse) vis.getDisplay(0);
		
		initializeGroupsFromAttributes();
	
	}
	
	public GroupsManager() {
		//the null groups manager.  Only used to display an empty table.
		vis = null;
	}

	private void initializeGroupsFromAttributes() {
		CyAttributes nodeAttributes = Cytoscape.getNodeAttributes();
		Iterator nodesIt = vis.getVisualGroup(CytoPrefuse.NODES).tuples();
		while (nodesIt.hasNext()){
			VisualItem node = (VisualItem)nodesIt.next();
			String id = Cytoscape.getCurrentNetwork().getNode(node.getInt(CytoPrefuse.CYTOSCAPE_ID)).getIdentifier(); 
			String groupMembers = nodeAttributes.getStringAttribute(id, ATTRIBUTE_NAME);					
			assignToGroups(node, groupMembers);
		}
		
		CyAttributes edgeAttributes = Cytoscape.getEdgeAttributes();
		Iterator edgesIt = vis.getVisualGroup(CytoPrefuse.EDGES).tuples();
		while (edgesIt.hasNext()){
			VisualItem edge = (VisualItem) edgesIt.next();
			String id = Cytoscape.getCurrentNetwork().getEdge(edge.getInt(CytoPrefuse.CYTOSCAPE_ID)).getIdentifier(); 
			String groupMembers = edgeAttributes.getStringAttribute(id, ATTRIBUTE_NAME);					
			assignToGroups(edge, groupMembers);	
		}
	}

	private void assignToGroups(VisualItem item, String groupMembers) {
		if (groupMembers != null){
			String[] groupNames = groupMembers.split(DELIMITER);
			for (int i=0; i < groupNames.length; i++){
				Group g = getOrCreateGroup(groupNames[i]);
				g.members.add(item);
			}
		}
	}

	private Group getOrCreateGroup(String groupName) {
		Iterator groupsIt = groups.iterator();
		while (groupsIt.hasNext()){
			Group g = (Group) groupsIt.next();
			if (g.name.equalsIgnoreCase(groupName)){
				return g;
			}
		}
		Group g = new Group(groupName);
		groups.add(g);
		return g;
		
	}

	public int getNumGroups() {
		return groups.size();
	}

	public String getGroupName(int groupID) {
		return  ((Group)groups.get(groupID)).name;
	}
	public String getCurrentGroupName (){
		return ((Group)groups.get(currentGroup)).name;
	}
	

	public void renameGroup(int groupID, String newName) {
		Group g = (Group) groups.get(groupID);
		g.name = newName;
		
		setCytoscapeGroupAttributes(g.members);				
	}

	public String addGroup() {
		boolean groupNameExists;			
		String newGroupName;
		do{
			newGroupName="Group " + (newGroupNumber ++);
		   groupNameExists = false;
		   Iterator groupsIt = groups.iterator();
		   while (groupsIt.hasNext()){
			   Group g = (Group)groupsIt.next();
			   if (g.name.equalsIgnoreCase(newGroupName)){
				   groupNameExists = true;
				   break;
			   }
		   }
	   }while (groupNameExists);
		groups.add(new Group(newGroupName));		
		fireTableRowsInserted(getNumGroups(),getNumGroups());
		return newGroupName;		
	}
	public void removeGroup(int selectedGroup) {
		Group g = (Group) groups.get(selectedGroup);
		groups.remove(selectedGroup);
		setCytoscapeGroupAttributes(g.members);
		fireTableRowsDeleted(selectedGroup, selectedGroup);
	}
	
	public void setCurrentGroup(int groupID, boolean isShiftDown){
		currentGroup = groupID;
        if (!isShiftDown){
            cytoPrefuse.deselectAll();
        }
		Group group = (Group) groups.get(currentGroup);
		Tuple[] selectedItems = new Tuple[group.members.size()];
		Iterator it = group.members.iterator();
		int i =0;
		while (it.hasNext()){
			selectedItems[i++] = (Tuple) it.next();
		}
        cytoPrefuse.selectInCytoscape(selectedItems, true);
	}
	
	public void addItemToGroup (VisualItem item){
		List singleItem = new ArrayList();
		singleItem.add(item);		
		addItemsToGroup(singleItem);
	}
	public void addItemToGroup(VisualItem item, String group) {
		List singleItem = new ArrayList();
		singleItem.add(item);		
		addItemsToGroup(singleItem, group);
	}
	public void addItemsToGroup (Collection items, String group){
		Group g = getGroup(group);
		addItemsToGroup(items, g);
	}
	
	public void addItemsToGroup(Collection items){
		Group g = (Group) groups.get(currentGroup);
		addItemsToGroup(items,g);
	}
	
	private void addItemsToGroup (Collection items, Group g){
		g.members.addAll(items);
		
		setCytoscapeGroupAttributes (items);		
	}
	
	private void setCytoscapeGroupAttributes(Collection items) {
		CyAttributes nodeAttributes = Cytoscape.getNodeAttributes();		
		CyAttributes edgeAttributes = Cytoscape.getEdgeAttributes();
		Iterator it = items.iterator();
		while (it.hasNext()){
			VisualItem item = (VisualItem)it.next();
			CyAttributes attributes = null;
			String id = "";
			if (item instanceof NodeItem){
				attributes = nodeAttributes;
				id = Cytoscape.getCurrentNetwork().getNode(item.getInt(CytoPrefuse.CYTOSCAPE_ID)).getIdentifier(); 
			}else if (item instanceof EdgeItem){
				attributes = edgeAttributes;
				id = Cytoscape.getCurrentNetwork().getEdge(item.getInt(CytoPrefuse.CYTOSCAPE_ID)).getIdentifier(); 				
			}
			
			String groupMembers = new String();
			Iterator groupsIt = groups.iterator();
			while (groupsIt.hasNext()){
				Group g = (Group)groupsIt.next();
				if (g.members.contains(item)){
					if (groupMembers.length() > 0){
						groupMembers += DELIMITER;
					}
					groupMembers += g.name;
				}
			}
			attributes.setAttribute(id, ATTRIBUTE_NAME, groupMembers);					

			reselectToCauseRefresh(item);
			
		}		
		vis.run("applyVizMapper");
	}
	
	private void reselectToCauseRefresh(VisualItem item) {
        if (item.isInGroup(CytoPrefuse.SELECTED)){
            		//Attribute browser doesn't refresh on setAttribute fire.  Fix that here.
			CytoPrefuse cytoPrefuse = (CytoPrefuse)vis.getDisplay(0);
			selectItem[0] = item;
			cytoPrefuse.selectInCytoscape(selectItem, false);
			cytoPrefuse.selectInCytoscape(selectItem, true);
		}
	}

	public void removeItemsFromGroup (Collection items){
		Group g = (Group) groups.get(currentGroup);
		g.members.removeAll(items);	

		setCytoscapeGroupAttributes(items);;
	}


	public void removeItemFromGroup (VisualItem item){
		List singleItem = new ArrayList();
		singleItem.add(item);		
		removeItemsFromGroup(singleItem);
	}
	
	private Group getGroup(String group) {
		Iterator it = groups.iterator();
		while (it.hasNext()){
			Group g = (Group)it.next();
			if (g.name.equalsIgnoreCase(group))
				return g;
		}
		return null;
	}

	private class Group{
		public String name;
		public Set members;
		public Group (String name){
			this.name = name;
			members = new HashSet();
		}
	}
	
	
	
	
	public int getColumnCount() {			
		return 1;
	}

	public int getRowCount() {
		return getNumGroups();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return getGroupName(rowIndex);
	}
	public boolean isCellEditable(int row, int col) {
		  return true;
	 }
	public Class getColumnClass(int c) {
		return String.class;
	}
	public void setValueAt(Object value, int row, int col) {
		  renameGroup(row, (String)value);
		  fireTableCellUpdated(row, col);
	}
	
	public String getColumnName(int col) {
		return "Groups";
	}


	

	
			
}
