package prefuse.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.controls.AbstractZoomControl;
import prefuse.controls.Control;
import prefuse.util.GraphicsLib;
import prefuse.util.ui.UILib;
import prefuse.visual.VisualItem;

/**
 * Zooms the display, changing the scale of the viewable region. By default,
 * zooming is achieved by pressing the right mouse button on the background
 * of the visualization and dragging the mouse up or down. Moving the mouse up
 * zooms out the display around the spot the mouse was originally pressed.
 * Moving the mouse down similarly zooms in the display, making items
 * larger.
 *
 * Zooming is bounded so that the bounding group doesn't get lost by
 * zooming in too far or zooming out to a dot.
 *
 * @author Aaron Barsky
 * @author <a href="http://jheer.org">jeffrey heer</a>
 */
public class BoundedZoomControl extends AbstractZoomControl {
    
    private int yLast;
    private Point2D down = new Point2D.Float();
    private int button = RIGHT_MOUSE_BUTTON; 
    private String boundingGroup;
    private double minZoom;
    private double groupScreenPercentAtMin;
    private double maxZoom;
    
    
    public BoundedZoomControl(String boundingGroup) {
       this (boundingGroup, RIGHT_MOUSE_BUTTON);
    }
   
    public BoundedZoomControl(String boundingGroup, int mouseButton) {
    	this (boundingGroup, mouseButton, 0.75, 2.0);
    }
    
    public BoundedZoomControl(String boundingGroup,     		
			double groupScreenPercentAtMin, double maxZoom){
    	this (boundingGroup, RIGHT_MOUSE_BUTTON, groupScreenPercentAtMin, maxZoom);
    }

    
    /**
     * Create a new bounded zoom control.
     * @param boundingGroup The visual group that should always be visible.
     * @param mouseButton the mouse button that should initiate a zoom. One of
     * {@link Control#LEFT_MOUSE_BUTTON}, {@link Control#MIDDLE_MOUSE_BUTTON},
     * or {@link Control#RIGHT_MOUSE_BUTTON}.
     * @param groupScreenPercentAtMin When zoomed out as far as possible, the group will occupy this much of the screen.
     * @param maxZoom The maximum zoom level.
     */
  
    public BoundedZoomControl(String boundingGroup, int mouseButton,
    							double groupScreenPercentAtMin, double maxZoom)
    {
     	this.boundingGroup = boundingGroup;
    	button = mouseButton;
    	this.groupScreenPercentAtMin = groupScreenPercentAtMin;
    	this.maxZoom = maxZoom;
    }
    
    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e) {
        if ( UILib.isButtonPressed(e, button)  && !e.isAltDown() ) {
            Display display = (Display)e.getComponent();
            if (display.isTranformInProgress()) {
                yLast = -1;
                System.err.println("can't move");
                return;
            }
            display.setHighQuality(false);
            display.setCursor(
                Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
            //Simplify to Cytoscape model.  Instead of            
            //zooming to the mouse, zoom to the center of the screen
            down.setLocation(display.getWidth()/2, (display.getHeight())/2);
            //  display.getAbsoluteCoordinate(e.getPoint(), down);
             
            yLast = e.getY();
            
            //Find the limits of the zoom
            Visualization vis = display.getVisualization();
            Rectangle2D bounds = vis.getBounds(boundingGroup);
            GraphicsLib.expand(bounds, 50 + (int)(1/display.getScale()));
            
            double w = display.getWidth(), h = display.getHeight();
            double cx = bounds.getCenterX();
            double cy =  bounds.getCenterY();
            double wb = Math.max(cx-bounds.getMinX(),
                                 bounds.getMaxX()-cx);
            double hb = Math.max(cy-bounds.getMinY(),
                                 bounds.getMaxY()-cy);
            
            minZoom = Math.min(w/(2*wb),h/(2*hb));
     
            minZoom =  minZoom * groupScreenPercentAtMin;
        }
    }
    
    /**
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e) {
        if ( UILib.isButtonPressed(e, button)&& !e.isAltDown() ) {
            Display display = (Display)e.getComponent();
            if (display.isTranformInProgress() || yLast == -1) {
                yLast = -1;
                return;
            }
            
           
            int y = e.getY();
            int dy = yLast - y;
            double zoom = 1 + ((double)dy) / 100;
            
            double afterZoomScale = display.getScale() * zoom;
            if (afterZoomScale> maxZoom)
            	zoom = maxZoom/display.getScale();
            else if (afterZoomScale < minZoom){
            	zoom = minZoom/display.getScale();
            }
            
            int status = zoom(display, down, zoom, false);
            int cursor = Cursor.N_RESIZE_CURSOR;
            if ( status == NO_ZOOM )
                cursor = Cursor.WAIT_CURSOR;
            display.setCursor(Cursor.getPredefinedCursor(cursor));
            
            yLast = y;
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e) {
        if ( UILib.isButtonPressed(e, button) ) {
        	 Display display = (Display)e.getComponent();
             display.setHighQuality(true);
             display.setCursor(Cursor.getDefaultCursor());
             display.repaint();
        }
    }
    
    /**
     * @see prefuse.controls.Control#itemPressed(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemPressed(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mousePressed(e);
    }

    /**
     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemDragged(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mouseDragged(e);
    }
    
    /**
     * @see prefuse.controls.Control#itemReleased(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemReleased(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mouseReleased(e);
    }
    
} // end of class ZoomControl
